#
# Basis Encoding for full DATA and MOTI AGCT
#
#
# called from external: quantum_ram_2qubits_basis_encode for SNPs
#
#
#


from qiskit.circuit.library import RYGate
from qiskit.visualization import circuit_drawer

from qiskit import QuantumCircuit, transpile, assemble
from qiskit_aer import Aer, AerSimulator
from qiskit.visualization import plot_histogram
import matplotlib.pyplot as plt


import matplotlib.pyplot  as plt
import numpy as np
import math
import csv

from util import logging_utils

logger = logging_utils.setup_logger('qc_logger', 'qc_log.log')


def index_to_qubit_states(index, num_qubits):
    binary_index = bin(index)[2:].zfill(num_qubits)
    return [int(bit) for bit in binary_index]

def flip_qubits(qc, qubits_states):
    for i, state in enumerate(qubits_states):
        if state == 1:
            qc.x(i)

def apply_controlled_not_gate(qc, control_qubits, target_qubit):
    qc.mcx(control_qubits, target_qubit)


def qcm_basis_basis (qc, data1, data2, num_qubits):


    state_vector_size = 2 ** num_qubits

    ## SUPERPOSTION DIRECT ADDRESSING CONTROL 
    for i in range(num_qubits - 2): 
            qc.h(i)

    print (qc)


    """ Process data1 and data2 using quantum gates """
    #for i in range(len(data1)):
    for i in range(len(data1)):
        qubits_index = index_to_qubit_states(i, num_qubits - 2)

        if data1[i] == '1' or data2[i] == '1':
            print(f"Data Index {i}: Data1={data1[i]}, Data2={data2[i]}")
            flip_qubits(qc, qubits_index)

            if data1[i] == '1':
                apply_controlled_not_gate(qc, list(range(num_qubits - 2)), num_qubits - 1)
            
            if data2[i] == '1':
                 apply_controlled_not_gate(qc, list(range(num_qubits - 2)), num_qubits - 2)
            flip_qubits(qc, qubits_index)
            
            qc.barrier()

    print(qc)

    return qc



###
###  Public Interface for XOR
###

def quantum_ram_basis_and_basis (data1, data2):
    # Calculate the number of qubits needed for addressing based on the length of the encoding data
    # Qaddr = log base 2 N
    num_address_qubits = int(math.ceil(math.log(len(data1), 2))) 

    # 2 qubits for (data1 and data2)
    total_qubits = num_address_qubits + 2

    print (f"data len = {len(data1)}")

    # Create a quantum circuit with the required number of qubits
    qc = QuantumCircuit(total_qubits)

    print(qc)

    # Define indices for the data1 and data2 qubits
    data1_indices = {'data1': total_qubits - 1 }
    data2_indices = {'data2': total_qubits - 2 }

    print (f'data1 qubit = {data1_indices}')
    print (f'data2 qubit = {data2_indices}')

    qcm_basis_basis (qc, data1, data2, total_qubits)

    return qc, total_qubits

