# main.py

from util import logging_utils
from hybrid_encoding import quantum_ram_basis_and_basis
from compare import quantum_compare
from qiskit import QuantumCircuit, transpile, assemble
from qiskit_aer import Aer, AerSimulator
from qiskit.visualization import plot_histogram
import matplotlib.pyplot as plt

import time
import datetime
import numpy as np
import matplotlib.pyplot  as plt
import math
import csv

logger = logging_utils.setup_logger('qc_logger', 'qc_log.log')



def main():

    platform = 1

    if platform == 0: #windows
        data_path = "D:\source\KMUTNB\Qnx\data\data.dat"
       
    else: #linux
        data_path = "/Users/we/code/qlab/lab1/src/hybrid/data.dat"
   

def load_and_check_data(file_path):
    # Load the data file
    try:
        with open(file_path, "r") as file:
            lines = file.readlines()
    except Exception as e:
        print(f"Error loading data file: {e}")
        return None, None

    # Ensure there are at least two lines in the file
    if len(lines) < 2:
        print("Invalid data format. Expecting two lines for Data1 and Data2.")
        return None, None

    # Extract data1 and data2 from lines
    try:
        data1 = lines[0].strip().split('=')[1]  # Extract binary string from Data1
        data2 = lines[1].strip().split('=')[1]  # Extract binary string from Data2
    except IndexError:
        print("Error parsing data file. Ensure the format is 'Data1=...' and 'Data2=...'")
        return None, None

    # Print sizes
    print(f"Size of data1: {len(data1)}")
    print(f"Size of data2: {len(data2)}")

    return data1, data2


def main():
    data_path = "data.dat"  # Ensure correct path to the data file
    data1, data2 = load_and_check_data(data_path)
    
    print (f"data1: {data1}")
    print (f"data2: {data2}")

    if data1 is not None and data2 is not None:
        print("Data successfully loaded and checked.")
    else:
        print("Failed to load data correctly.")

    qc, total = quantum_ram_basis_and_basis(data1, data2)

    print("State Encoding")
    #print(qc)

    quantum_compare(qc, total-1, total-2)

    qc.measure_all()

    print(qc)

    #For execution
    simulator = AerSimulator()
    #compiled_circuit = transpile(qc, simulator)
    sim_result = simulator.run(qc).result()
    counts = sim_result.get_counts()


    #qc.draw(output='mpl')
    #plt.show()

    # Step 6: Get and display the result as a histogram
    print("Result:", counts)
    #plot_histogram(counts)
    #plt.show()

    interpret_results(qc, sim_result, len(data1))


def interpret_results(qc, results, data_length):
   
    total_qubits = qc.num_qubits
    num_address_qubits = int(np.ceil(np.log2(data_length)))
    num_data_qubits = 2

    histogram_data = results.get_counts(qc)


    decoded_results = []

    for state, count in histogram_data.items():
        databits = state[:num_data_qubits]  # First 2 bits are data bits
        address_bits = state[num_data_qubits:num_data_qubits + num_address_qubits]  # remaining bits are address bits

        # Flip and reverse address bits to match the original indexing
        flipped_address = ''.join('1' if bit == '0' else '0' for bit in address_bits)
        reversed_address = flipped_address[::-1]

        decimal_address = int(reversed_address, 2)

        decoded_results.append({
            "databits": databits,
            "address_bits": address_bits,
            "flipped_reversed_address": reversed_address,
            "decimal_address": decimal_address,
            "count": count
        })

    # Sort results by decimal_address
    decoded_results_sorted = sorted(decoded_results, key=lambda x: x["decimal_address"])

    print("\nDecoded Quantum Results (Sorted by Address):")
    for result in decoded_results:
        print(f"Address (Decimal): {result['decimal_address']}, Data bits: {result['databits']}, "
              f"Flipped & Reversed: {result['flipped_reversed_address']}, Count: {result['count']}")

if __name__ == "__main__":
    main()


