import logging

def setup_logger(name, log_file, level=logging.INFO):

    logger = logging.getLogger(name)

    # Set the logging level.
    logger.setLevel(level)

    # Create a file handler.
    file_handler = logging.FileHandler(log_file)

    # Create a stream handler.
    stream_handler = logging.StreamHandler()

    # Create a formatter.
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

    # Add the handlers and formatter to the logger.
    file_handler.setFormatter(formatter)
    stream_handler.setFormatter(formatter)
    logger.addHandler(file_handler)
    logger.addHandler(stream_handler)

    return logger


def log_message(logger, message, level=logging.INFO):
    logger.log(level, message)

def log_info(logger, message):
    logger.info(message)

def log_warning(logger, message):
    logger.warning(message)

def log_error(logger, message):
    logger.error(message)

def log_critical(logger, message):
    logger.critical(message)



#You can also use the `log_warning()`, `log_error()`, and `log_critical()` functions to log warning, error, and critical messages, respectively.
#By default, the `setup_logger()` function will create a logger that logs messages to both a file and the console. You can change this behavior by specifying the `stream=False` or `file=False` arguments to the `setup_logger()` function. For example, the following code creates a logger that only logs messages to a file:

#```python
#logger = logging_utils.setup_logger('my_logger', 'my_log.log', stream=False)
#```

#You can also use the `logging_utils` module to configure the logging level for a logger. The logging level determines which messages are logged. The following code sets the logging level for the `my_logger` logger to `logging.DEBUG`:

#```python
#logger = logging_utils.setup_logger('my_logger', 'my_log.log')
#logger.setLevel(logging.DEBUG)

